<?php

namespace Drupal\bakery\Forms;

use Drupal\bakery\BakeryService;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Contribute form.
 */
class BakeryPullForm extends FormBase implements ContainerInjectionInterface {

  /**
   * @var \Drupal\bakery\BakeryService
   */
  private $service;

  /**
   * @param \Drupal\bakery\BakeryService $service
   */
  public function __construct(BakeryService $service) {
    $this->service = $service;
  }

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *
   * @return \Drupal\bakery\Forms\BakeryPullForm|static
   */
  public static function create(ContainerInterface $container): BakeryPullForm|static {
    return new static($container->get('bakery.bakery_service'));
  }

  /**
   * {@inheritdoc}
   *
   * @return string
   */
  public function getFormId(): string {
    return 'bakery_forms_pull_form';
  }

  /**
   * Form for admins to pull accounts.
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array<string, mixed>
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['or_email'] = [
      '#type' => 'radios',
      '#options' => [
        0 => t('Username'),
        1 => t('Username or email'),
      ],
      '#default_value' => 0,
    ];

    $form['name'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Request account'),
    ];

    return $form;
  }

  /**
   * Make sure we are not trying to request an existing user.
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\user\UserInterface|false $existing_account */
    $existing_account = user_load_by_name(
      is_string($form_state->getValue('name')) ? $form_state->getValue('name') : '',
    );
    if (!$existing_account && $form_state->getValue('or_email')) {
      /** @var \Drupal\user\UserInterface|false $existing_account */
      $existing_account = user_load_by_mail(
        is_string($form_state->getValue('name')) ? $form_state->getValue('name') : '',
      );
    }
    // Raise an error in case the account already exists locally.
    if ($existing_account) {
      $form_state->setError($form['name'], $this->t('Account @link exists.', [
        '@link' => $existing_account->toLink($existing_account->getAccountName()),
      ]));
    }
  }

  /**
   * If the request succeeds, go to the user page. O5:1otherwise, show an error.
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $result = $this->service->requestAccount(
        is_string($form_state->getValue('name')) ? $form_state->getValue('name') : '',
        is_bool($form_state->getValue('or_email')) ? $form_state->getValue('or_email') : false
    );

    if ($result == '') {
      $this->messenger()->addError($this->t("Pulling account %name failed: maybe there is a typo or they don't exist on the master site.", [
        '%name' => $form_state->getValue('name'),
      ]));
    }
    else {
      $form_state->setRedirect('user', ['id' => $result]);
    }
  }

}
