<?php

namespace Drupal\bakery\Forms;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure bakery settings for this site.
 */
class BakerySettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bakery_admin_settings';
  }

  /**
   * {@inheritdoc}
   *
   * @return string[]
   */
  protected function getEditableConfigNames(): array {
    return [
      'bakery.settings',
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array<string, mixed>
   * @throws \Random\RandomException
   *
   */
  public function buildForm(array $form, FormStateInterface $form_state) : array {

    $config = $this->config('bakery.settings');

    $form['bakery_is_parent'] = [
      '#type' => 'checkbox',
      '#title' => 'Is this the parent site?',
      '#default_value' => $config->get('bakery_is_parent'),
      '#description' => $this->t('On the master site, accounts need to be created by traditional processes, i.e by a user registering or an admin creating them.'),
    ];

    $form['bakery_parent'] = [
      '#type' => 'textfield',
      '#title' => 'Master site',
      '#default_value' => $config->get('bakery_parent'),
      '#description' => $this->t('Specify the parent site for your bakery network.'),
    ];


    $bakery_children = $config->get('bakery_children');
    if(is_array($bakery_children)){
      $bakery_children = implode("\n", $bakery_children);
    }

    $form['bakery_children'] = [
      '#type' => 'textarea',
      '#title' => 'Children sites',
      '#default_value' =>  $bakery_children,
      '#description' => $this->t('Specify any children sites in your bakery network that you want to update if a user changes email or username on the master. Enter one site per line, in the form "http://sub.example.com/".'),
    ];

    $form['bakery_help_text'] = [
      '#type' => 'textarea',
      '#title' => 'Help text for users with synch problems.',
      '#default_value' => $config->get('bakery_help_text'),
      '#description' => $this->t('This message will be shown to users if/when they have problems synching their accounts. It is an alternative to the "self repair" option and can be blank.'),
    ];

    $form['bakery_freshness'] = [
      '#type' => 'textfield',
      '#title' => 'Seconds of age before a cookie is old',
      '#default_value' => $config->get('bakery_freshness'),
    ];

    $form['bakery_key'] = [
      '#type' => 'textfield',
      '#title' => 'Private key for cookie validation',
      '#default_value' => $config->get('bakery_key'),
      '#description' => $this->t('This key needs to be at least 32 characters and will be used to validate cookies.'),
      '#placeholder' => $this->t('Example key: @key', ['@key' => bin2hex(random_bytes(32))])
    ];

    $form['bakery_domain'] = [
      '#type' => 'textfield',
      '#title' => 'Cookie domain',
      '#default_value' => $config->get('bakery_domain'),
    ];


    $default = $config->get('bakery_supported_fields');
    if (!is_array($default)) {
      $default = [];
    }
    $default['mail'] = 'mail';
    $default['name'] = 'name';
    $options = [
      'name' => $this->t('username'),
      'mail' => $this->t('e-mail'),
      'status' => $this->t('status'),
      'picture' => $this->t('user picture'),
      'language' => $this->t('language'),
      'signature' => $this->t('signature'),
    ];
    // TODO: need to add profile fileds
    /*
    if (module_exists('profile')) {
    $result = db_query('SELECT name, title FROM {profile_field}
    ORDER BY category, weight');
    foreach ($result as $field) {
    $options[$field->name] = check_plain($field->title);
    }
    }
     */
    $form['bakery_supported_fields'] = [
      '#type' => 'checkboxes',
      '#title' => 'Supported profile fields',
      '#default_value' => $default,
      '#options' => $options,
      '#description' => $this->t('Choose the profile fields that should be exported by the master and imported on the children. Username and E-mail are always exported. The correct export of individual fields may depend on the appropriate settings for other modules on both parent and children. You need to configure this setting on both the parent and the children.'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('bakery.settings');

    $form_state_parent = $form_state->getValue('bakery_parent');
    if(!is_string($form_state_parent)){
      $form_state_parent = '';
    } else {
      $form_state_parent = trim($form_state_parent, '/') . '/';
    }

    $config
      ->set('bakery_is_parent', $form_state->getValue('bakery_is_parent'))
      ->set('bakery_parent', $form_state_parent)
      ->set('bakery_help_text', $form_state->getValue('bakery_help_text'))
      ->set('bakery_freshness', $form_state->getValue('bakery_freshness'))
      ->set('bakery_key', $form_state->getValue('bakery_key'))
      ->set('bakery_domain', $form_state->getValue('bakery_domain'))
      ->set('bakery_supported_fields', $form_state->getValue('bakery_supported_fields'));
    $children = $form_state->getValue('bakery_children');
    if(!is_string($children)){
      throw new \UnexpectedValueException( 'BSFSF');
    }

    if ($children) {
      // Transform the text string into an array.
      $slaves = explode("\n", trim(str_replace("\r", '', $children)));
      // For each entry, remove the trailing slash
      // (if present) and concatenate with a new trailing slash.
      foreach ($slaves as &$slave) {
        $slave = trim($slave, '/') . '/';
      }
      $config->set('bakery_children', $slaves);
    }
    else {
      $config->set('bakery_children', []);
    }
    $config->save();
    // Trigger some rebuilds that might be required by main/child changes.
    \Drupal::service('router.builder')->setRebuildNeeded();

    parent::submitForm($form, $form_state);
  }

}
