<?php

namespace Drupal\bakery\Forms;

use Drupal\bakery\Kitchen;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Password\PasswordInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Contribute form.
 */
class BakeryUncrumbleForm extends FormBase implements ContainerInjectionInterface {

  /**
   * For generating hashed password we check with database.
   *
   * @var \Drupal\Core\Password\PasswordInterface
   */
  private PasswordInterface $passwordHasher;

  /**
   * @var \Drupal\Core\Database\Connection
   */
  private Connection $database;

  /**
   * @var \Drupal\bakery\Kitchen
   */
  private Kitchen $kitchen;

  /**
   * Class constructor.
   */
  public function __construct(PasswordInterface $password_hasher, Connection $connection, Kitchen $kitchen) {
    // TODO some sort of session storage abstraction?
    $this->passwordHasher = $password_hasher;
    $this->database = $connection;
    $this->kitchen = $kitchen;
  }

  /**
   * {@inheritdoc}
   *
   * @return object
   */
  public static function create(ContainerInterface $container): object {
    return new static(
      $container->get('password'),
      $container->get('database'),
      $container->get('bakery.kitchen')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @return string
   */
  public function getFormId(): string {
    return 'bakery_forms_uncrumble_form';
  }

  /**
   * Form to let users repair minor problems themselves.
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array<string, mixed> $array of form elements.
   * @throws \Exception
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $cookie = $this->kitchen->taste(Kitchen::CHOCOLATE_CHIP);
    // Analyze.
    $query = $this->database->select('users_field_data', 'u')
      ->fields('u', ['uid', 'name', 'mail'])
      ->condition('u.uid', '0', '!=')
      ->condition('u.mail', '', '!=')
      ->where("LOWER(u.mail) = LOWER(:mail)", [':mail' => $cookie['mail'] ?? null]);
    $result = $query->execute();
    $same_mail = null;
    if ($result !== null) {
      /** @var \stdClass|null $same_mail */
      $same_mail = $result->fetchObject();
    }

    $query = $this->database->select('users_field_data', 'u')
      ->fields('u', ['uid', 'name', 'mail'])
      ->condition('u.uid', '0', '!=')
      ->where("LOWER(u.name) = LOWER(:name)", [':name' => $cookie['name'] ?? null]);
    $result = $query->execute();
    $same_name = null;
    if ($result !== null) {
      /** @var \stdClass|null $same_name */
      $same_name = $result->fetchObject();
    }

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#value' => $cookie['name'] ?? null,
      '#disabled' => TRUE,
      '#required' => TRUE,
    ];

    $form['mail'] = [
      '#type' => 'item',
      '#title' => t('Email address'),
      '#value' => $cookie['mail'] ?? null,
      '#required' => TRUE,
    ];

    $form['pass'] = [
      '#type' => 'password',
      '#title' => t('Password'),
      '#description' => t('Enter the password that accompanies your username.'),
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Repair account'),
      '#weight' => 2,
    ];

    $help = '';
    $init_cookie = $cookie['init'] ?? null;
    $result_uid = $this->database->select('users_field_data', 'u')
      ->fields('u', ['uid'])
      ->condition('init', [$init_cookie] , '=')
      ->countQuery()
      ->execute();

    $count = null;
    if ($result_uid !== null) {
      $count = $result_uid->fetchField();
    }

    if ($count > 1) {
      $this->messenger()->addMessage(t('Multiple accounts are associated with your master account. This must be fixed manually. <a href="@contact">Please contact the site administrator.</a>', [
        '%email' => $cookie['mail'] ?? null,
        '@contact' => $this->configFactory()->get('bakery.settings')->get('bakery_parent') . 'contact',
      ]));
      $form['pass']['#disabled'] = TRUE;
      $form['submit']['#disabled'] = TRUE;
    }
    elseif ($same_name && $same_mail && $same_name->uid != $same_mail->uid) {
      $this->messenger()->addMessage(t('Both an account with matching name and an account with matching email address exist, but they are different accounts. This must be fixed manually. <a href="@contact">Please contact the site administrator.</a>', [
        '%email' => $cookie['mail'] ?? null,
        '@contact' => $this->configFactory()->get('bakery.settings')->get('bakery_parent') . 'contact',
      ]));
      $form['pass']['#disabled'] = TRUE;
      $form['submit']['#disabled'] = TRUE;
    }
    elseif ($same_name) {
      $site_name = $this->configFactory()->get('system.site')->get('name');
      $help = t("An account with a matching username was found. Repairing it will reset the email address to match your master account. If this is the correct account, please enter your %site password.", ['%site' => $site_name]);
      // This is a borderline information leak.
      // $form['mail']['#value'] = $samename->mail;.
      $form['mail']['#value'] = t('<em>*hidden*</em>');
      $form['mail']['#description'] = t('Will change to %new.', ['%new' => $cookie['mail'] ?? null]);
    }
    elseif ($same_mail) {
      $site_name = $this->configFactory()->get('system.site')->get('name');
      $help = t("An account with a matching email address was found. Repairing it will reset the username to match your master account. If this is the correct account, please enter your %site password.", ['%site' => $site_name]);
      /** @var object|null $same_mail */
      $form['name']['#value'] = $same_mail->name ?? null;
      $form['name']['#description'] = t('Will change to %new.', ['%new' => $cookie['name'] ?? null]);
    }

    $form['help'] = ['#weight' => -10, '#markup' => $help];

    return $form;
  }

  /**
   * Validation for bakery_uncrumble form.
   *
   * @param array<string, mixed> $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    /*
     * @todo review this comment
     * We are ignoring blocked status on purpose.
     * The user is being repaired, not logged in.
     */
    /** @var string $name */
    $name = $form_state->getValue('name');
    /** @var \Drupal\user\Entity\User $account */
    $account = user_load_by_name($name);
    /** @var string $pass */
    $pass = $form_state->getValue('pass');
    $hash = $account->getPassword();
    if (!($account->id()) || ($hash !== null && $this->passwordHasher->check($pass, $hash))) {
      $this->logger('bakery')
        ->notice('Login attempt failed for %user while running uncrumble.', [
          '%user' => $form_state->getValue('name'),
        ]);
      /*
       * Can't pretend that it was the "username or password"
       * so let's be helpful instead.
       */
      $form_state->setErrorByName('pass', t('Sorry, unrecognized password. If you have forgotten your %site password, please <a href="@contact">contact the site administrator.</a>', [
        '%site' => $this->configFactory()->get('system.site')->get('name'),
        '@contact' => $this->configFactory()->get('bakery.settings')->get('bakery_parent') . 'contact',
      ]));

    }
    else {
      $form_state->setValue('bakery_uncrumble_account', $account);
    }
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\user\Entity\User $account */
    $account = $form_state->getValue('bakery_uncrumble_account');
    $form_state->unsetValue('bakery_uncrumble_account');
    $cookie = $this->kitchen->taste(Kitchen::CHOCOLATE_CHIP);
    $this->database->update('users_field_data')->fields(['init' => $cookie['init'] ?? null] )
      ->condition('uid', $account->id(), '=')
      ->execute();
    $logger = $this->logger('bakery');
    $logger->notice('uncrumble changed init field for uid %uid from %oldinit to %newinit', [
      '%oldinit' => $account->get('init')->value,
      '%newinit' => $cookie['init'] ?? null,
      '%uid' => $account->id(),
    ]);

    $email_cookie = $cookie['mail'] ?? '';
    if(!is_string($email_cookie)){
      throw new \UnexpectedValueException('String issue: BUFSF.');
    }
    $account->setEmail($email_cookie);
    $account->save();
    $logger
      ->notice('uncrumble updated name %name_old to %name_new, mail %mail_old to %mail_new on uid %uid.', [
        '%name_old' => $account->getAccountName(),
        '%name_new' => $cookie['name'] ?? '',
        '%mail_old' => $account->getEmail() ?? '',
        '%mail_new' => $cookie['mail'] ?? '',
        '%uid' => $account->id(),
      ]);
    $this->messenger()->addMessage(t('Your account has been repaired.'));
    $form_state->setRedirect('user.page');
  }

  /**
   * Only let people with actual problems mess with uncrumble.
   *
   * @param \Symfony\Component\HttpFoundation\Request|null $request
   *
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultForbidden
   */
  public function uncrumbleAccess(Request $request = NULL) {
    // @todo Work around https://www.drupal.org/project/drupal/issues/2786941
    $request = $request ?? \Drupal::request();
    if ($this->currentUser()->isAnonymous() && $request->getSession()->get('BAKERY_CRUMBLED', FALSE)) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

}
