<?php

namespace Drupal\Tests\bakery\Functional;

use Drupal\bakery\Cookies\ChocolateChip;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * @group bakery
 */
class ChildLoginTest extends ChildSiteTestBase {

  /**
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * @var \Drupal\user\Entity\User|false
   */
  protected $account;

  /**
   * @var string
   */
  protected $cookie_name;

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp(): void {
    parent::setUp();
    $this->account = $this->createUser(['access user profiles']);
    $this->cookie_name = ChocolateChip::getName();
  }

  /**
   * Creates the account array.
   *
   * @return array
   */
  protected function acc(): array {
    // Account Object
    $account = $this->account;

    // Account variables array.
    return [
      'id' => $account->id(),
      'name' => $account->getAccountName(),
      'email' => $account->getEmail(),
      'init_email' => $account->getInitialEmail(),
    ];
  }

  /**
   * Tests different types of child site logins.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testChildLogins() {

    // Get details of the account.
    $acc = $this->acc();

    // Create active session.
    $assert_session = $this->assertSession();

    // Bake a cookie.
    $this->bakeCookie(
      new ChocolateChip($acc['name'], $acc['email'], $acc['init_email'], '0')
    );

    // Test user exists on page.
    $this->drupalGet('/user/'. $acc['id']);
    $assert_session->responseContains($acc['name']);

    // Miss-matched username should not log in.
    $this->bakeCookie(
      new ChocolateChip(
        $acc['name'] . 'bad', $acc['email'], $acc['init_email'], '0')
    );
    $this->drupalGet('/user/'. $acc['id']);
    $assert_session->responseNotContains($acc['name']);

    // Miss-matched email shouldn't log in.
    $this->bakeCookie(
      new ChocolateChip(
        $acc['name'], $acc['email'] . 'bad' , $acc['init_email'], '0')
    );
    $this->drupalGet('/user/'. $acc['id']);
    $assert_session->responseNotContains($acc['name']);

    // Set a cookie signed by the main site and matching a user. Should log in.
    $this->bakeCookie(
      new ChocolateChip(
        $acc['name'], $acc['email'], $acc['init_email'], '1')
    );
    $this->drupalGet('');
    $assert_session->responseContains($acc['name']);

    //  Removing the chocolate chip cookie should boot the user.
    $this->eatCookie(ChocolateChip::getName());
    $this->drupalGet('/user/'. $acc['id']);
    $assert_session->responseNotContains($this->cookie_name);

    // See if a mangled init field is repaired on bakery login.
    $valid_init = 'drupal.org/user/123/edit';
    $this->account->set('init', 'bakery_temp/' . mt_rand())->save();
    $this->bakeCookie(new ChocolateChip($this->account->getAccountName(), $this->account->getEmail(), $valid_init, '1'));
    $this->drupalGet('');
    $assert_session->responseContains($this->account->getAccountName());
    $this->account = User::load($this->account->id());
    $this->assertEquals($valid_init, $this->account->getInitialEmail());
  }

  /**
   * Testing logout using cookie
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testChildLoginOut() {
    $acc = $this->acc();
    $assert_session = $this->assertSession();
    // Set a cookie signed by the main site and matching a user. Should log in.
    $this->bakeCookie(
      new ChocolateChip(
        $acc['name'], $acc['email'], $acc['init_email'], '0')
    );
    $this->drupalGet('');
    $assert_session->responseContains($acc['name']);
    $this->drupalLogout();
    $assert_session->responseNotContains($acc['name']);
  }

  /**
   * @todo not sure this is needed.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function testAccountCreation() {
    $this->markTestIncomplete('This can not currently work during a request because it can not talk to a main site.');

    $assert_session = $this->assertSession();
    $valid_init = 'drupal.org/user/123/edit';

    // Miss-matched username should not log in.
    $this->bakeCookie(new ChocolateChip('tester', 'test@example.com', $valid_init, '1'));
    $this->drupalGet('');

    $assert_session->responseContains('Tester');
    /** @var \Drupal\user\UserStorageInterface $user_storage */
    $user_storage = $this->container->get('entity_type.manager')->getStorage('user');
    $user = $user_storage->loadByProperties(['name' => 'tester']);
    $this->assertEquals('test@example.com', $user[0]->getEamil());
    $this->assertEquals($valid_init, $user[0]->getInitialEmail());
  }

  /**
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testLoginPage() {
    $this->drupalGet('user');
    $assert_session = $this->assertSession();
    $assert_session->fieldNotExists('name');
    $assert_session->fieldNotExists('pass');
    $assert_session->buttonNotExists('edit-submit');
    $assert_session->linkByHrefExists('?' . UrlHelper::buildQuery(['bd' => Url::fromRoute('user.login')->setAbsolute(TRUE)->toString()]));
  }

}
