<?php

namespace Drupal\Tests\bakery\Functional;

use Drupal\bakery\Cookies\ChocolateChip;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Url;

/**
 * @group bakery
 */
class MainSiteCookiesTest extends MainSiteTestBase {

  /**
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * @var \Drupal\user\Entity\User|false
   */
  protected $account;

  /**
   * @var string
   */
  protected $cookie_name;

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp(): void {
    parent::setUp();
    $this->account = $this->createUser(['access user profiles']);
    $this->cookie_name = ChocolateChip::getName();
  }

  /**
   * Test login for the main (parent) site.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testMainLoginUsingCookies() {

    $cookie_name = $this->cookie_name;
    $account = $this->account;
    $assert_session = $this->assertSession();

    // Miss-matched username should not log in.
    $this->bakeCookie(new ChocolateChip($account->getAccountName() . 'bad', $account->getEmail(), $account->getInitialEmail(), '1'));
    $this->drupalGet('');
    $assert_session->responseNotContains($account->getAccountName());

    // Miss-matched email shouldn't log in.
    $this->bakeCookie(new ChocolateChip($account->getAccountName(), 'bad' . $account->getEmail(), $account->getInitialEmail(), '1'));
    $this->drupalGet('');
    $assert_session->responseNotContains($account->getAccountName());

    // Even valid cookies don't log a use in.
    $this->bakeCookie(new ChocolateChip($account->getAccountName(), $account->getEmail(), $account->getInitialEmail(), '1'));
    $this->drupalGet('');
    $assert_session->responseNotContains($account->getAccountName());
    $this->eatCookie($cookie_name);

    $this->drupalLogin($account);
    // Username mismatch should kill the session.
    $this->bakeCookie(new ChocolateChip($account->getAccountName() . 'bad', $account->getEmail(), $account->getInitialEmail(), '1'));
    $this->drupalGet('/user');
    $assert_session->responseNotContains($account->getAccountName());
    // Cleanup logged in user stuff. Basically reset state like drupalLogout.
    $this->eatCookie($cookie_name);
  }

  /**
   * @return void
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testMainLoginUsingCookiesA() {

    $cookie_name = $this->cookie_name;
    $account = $this->account;
    $assert_session = $this->assertSession();

    $this->drupalGet(Url::fromRoute('user.login'));
    $this->submitForm([
      'name' => $account->getAccountName(),
      'pass' => $account->passRaw . 'badpass',
    ], 'Log in');
    $account->sessionId = $this->getSession()->getCookie(\Drupal::service('session_configuration')->getOptions(\Drupal::request())['name']);
    $this->assertFalse($this->drupalUserIsLoggedIn($account), new FormattableMarkup('User %name successfully logged in.', ['%name' => $account->getAccountName()]));
    $assert_session->assert($this->getSession()->getCookie($cookie_name) === null, 'Cookie is set but should not be.');

    $this->drupalLogin($account);
    $assert_session->cookieExists($cookie_name);
    $this->assertCookieTastesGood($this->getSession()->getCookie($cookie_name), $cookie_name);
    // Without bakery cookie, user gets logged back out.
    $this->eatCookie($cookie_name);
    $this->drupalGet('');
    $assert_session->responseNotContains($account->getAccountName());

    /**
     * Test redirects...
     */
    $this->drupalGet(Url::fromRoute('user.login', [], ['query' => ['bd' => urlencode('https://www.google.org/')]]));
    $this->submitForm([
      'name' => $account->getAccountName(),
      'pass' => $account->passRaw . 'badpass',
    ], 'Log in');
    $account->sessionId = $this->getSession()->getCookie(\Drupal::service('session_configuration')->getOptions(\Drupal::request())['name']);
    $this->assertFalse($this->drupalUserIsLoggedIn($account), new FormattableMarkup('User %name successfully logged in.', ['%name' => $account->getAccountName()]));
    $assert_session->assert($this->getSession()->getCookie($cookie_name) === null, 'Cookie is set but should not be.');

    $this->drupalGet(Url::fromRoute('user.login', [], ['query' => ['bd' => urlencode('https://www.google.org/')]]));
    $this->submitForm([
      'name' => $account->getAccountName(),
      'pass' => $account->passRaw,
    ], 'Log in');
    $account->sessionId = $this->getSession()->getCookie(\Drupal::service('session_configuration')->getOptions(\Drupal::request())['name']);

    $this->drupalGet('');
    $assert_session->cookieExists($cookie_name);
    $this->assertCookieTastesGood($this->getSession()->getCookie($cookie_name), $cookie_name);
    $assert_session->responseContains($account->getAccountName());
  }

}
